@@ -1,5 +1,6 @@
 #!/bin/sh
 
+# make-lang.sh v1.0.10
 # this script is used to create a language-specifi XPI for Enigmail
 
 # if you want to debug this script, set DEBUG to something >0
@@ -9,7 +10,7 @@ if [ $# -ne 2 ]; then
   echo "Usage: $0 xx-YY version"
   echo "       where: xx-YY   is the language and country code representing the"
   echo "                      translated language"
-  echo "              version is the Enigmail version, e.g. 0.76.8"
+  echo "              version is the Enigmail version, e.g. 0.84.1"
   exit 1
 fi
 
@@ -20,8 +21,11 @@ ENIGVERSION=$2
 export ENIGVERSION
 
 LANGDIR=${ENIGLANG}/locale/${ENIGLANG}/enigmail
+HELPDIR=${LANGDIR}/help
+cwd=`pwd`
 rm -rf ${LANGDIR} >/dev/null 2>&1
 mkdir -p ${LANGDIR} 
+mkdir -p ${HELPDIR}
 
 # create install.js
 cat > ${ENIGLANG}/install.js <<EOT
@@ -151,6 +155,50 @@ function getPlatform() {
 }
 EOT
 
+# create install.rdf for Thunderbird 0.7 and newer
+cat > ${ENIGLANG}/install.rdf <<EOT
+<?xml version="1.0"?>
+
+<RDF xmlns="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
+     xmlns:em="http://www.mozilla.org/2004/em-rdf#">
+
+  <Description about="urn:mozilla:install-manifest">
+    <em:id>{847b3a00-7ab0-11d4-8f02-006008948af5}</em:id>
+    <em:version>${ENIGVERSION}</em:version>
+    
+    <!-- Target Application (Thunderbird) this extension can install into, 
+         with minimum and maximum supported versions. --> 
+    <em:targetApplication>
+      <Description>
+        <em:id>{3550f703-e582-4d05-9a08-453d09bdfdc6}</em:id>
+        <em:minVersion>0.6</em:minVersion>
+        <em:maxVersion>1.0</em:maxVersion>
+      </Description>
+    </em:targetApplication>
+    
+    
+    <!-- Front End MetaData -->
+    <em:name>Enigmail</em:name>
+    <em:description>${ENIGLANG} language pack for Enigmail</em:description>
+    <em:creator>Enigmail Team</em:creator>
+    <em:homepageURL>http://enigmail.mozdev.org/langpack.html</em:homepageURL>
+
+    <!-- Front End Integration Hooks (used by Extension Manager)-->
+    <em:optionsURL>chrome://enigmail/content/pref-enigmail.xul</em:optionsURL>
+    <em:aboutURL>chrome://enigmail/content/enigmailAbout.xul</em:aboutURL>
+    <em:iconURL>chrome://enigmail/skin/enigmail-about.png</em:iconURL>
+    
+    <!-- Packages, Skins and Locales that this extension registers -->
+    <em:file>
+      <Description about="urn:mozilla:extension:file:enigmail-${ENIGLANG}.jar">
+        <em:locale>locale/${ENIGLANG}/enigmail/</em:locale>
+      </Description>
+    </em:file>
+  </Description>      
+</RDF>
+
+EOT
+
 # create enigmail-xx-YY.spec
 cat > ${ENIGLANG}/enigmail-${ENIGLANG}.spec <<EOT
 Name:      mozilla-enigmail-${ENIGLANG}
@@ -242,12 +290,22 @@ cat >${LANGDIR}/contents.rdf <<EOT
 </RDF:RDF>
 EOT
 
-cp enigmail.dtd  ${LANGDIR}
-cp enigmail.properties ${LANGDIR}
+for f in enigmail.dtd enigmail.properties am-enigprefs.properties upgrade_080.html ; do
+  cp ${f} ${LANGDIR}
+done
 
-cd ${ENIGLANG}
+if [ -d help ]; then
+  cd help
+fi
+pwd
+
+for f in compose.html messenger.html rulesEditor.html editRcptRule.html ; do
+  cp ${f} ${cwd}/${HELPDIR} 
+done
+
+cd ${cwd}/${ENIGLANG}
 zip -r -D enigmail-${ENIGLANG}.jar locale
-zip ../enigmail-${ENIGLANG}-${ENIGVERSION}.xpi install.js enigmail-${ENIGLANG}.spec enigmail-${ENIGLANG}.jar
+zip ../enigmail-${ENIGLANG}-${ENIGVERSION}.xpi install.js install.rdf enigmail-${ENIGLANG}.spec enigmail-${ENIGLANG}.jar
 cd ..
 
 test $DEBUG -eq 0 && rm -rf ${ENIGLANG}
