@@ -34,8 +34,8 @@ GPL.
 // enigmailCommon.js: shared JS functions for Enigmail
 
 // This Enigmail version and compatible Enigmime version
-var gEnigmailVersion = "0.84.0.0";
-var gEnigmimeVersion = "0.84.0.0";
+var gEnigmailVersion = "0.84.2.0";
+var gEnigmimeVersion = "0.84.2.0";
 
 // Maximum size of message directly processed by Enigmail
 const ENIG_MSG_BUFFER_SIZE = 96000;
@@ -57,10 +57,14 @@ const ENIG_MIME_CONTRACTID = "@mozilla.org/mime;1";
 const ENIG_WMEDIATOR_CONTRACTID = "@mozilla.org/rdf/datasource;1?name=window-mediator";
 const ENIG_APPSHELL_MEDIATOR_CONTRACTID = "@mozilla.org/appshell/window-mediator;1"
 const ENIG_ASS_CONTRACTID = "@mozilla.org/appshell/appShellService;1";
+const ENIG_CLIPBOARD_CONTRACTID = "@mozilla.org/widget/clipboard;1";
+const ENIG_CLIPBOARD_HELPER_CONTRACTID = "@mozilla.org/widget/clipboardhelper;1"
+const ENIG_TRANSFERABLE_CONTRACTID = "@mozilla.org/widget/transferable;1"
 
 const ENIG_LOCALFILEOUTPUTSTREAM_CONTRACTID =
                               "@mozilla.org/network/file-output-stream;1";
 const ENIG_STANDARD_URL_CONTRACTID = "@mozilla.org/network/standard-url;1";
+const ENIG_SCRIPTABLEINPUTSTREAM_CONTRACTID = "@mozilla.org/scriptableinputstream;1"
 
 const ENIG_STREAMCONVERTERSERVICE_CID_STR =
       "{892FFEB0-3F80-11d3-A16C-0050041CAF44}";
@@ -340,7 +344,7 @@ function EnigConfigure() {
   var prefList = gPrefEnigmail.getChildList("",obj);
 
   for (var prefItem in prefList) {
-    prefName=prefList[prefItem];
+    var prefName=prefList[prefItem];
     if (prefName.search(/AlertCount$/) >= 0) {
        // Reset alert count to default value
       try {
@@ -371,7 +375,6 @@ const ENIG_CREATE_FILE = 0x08;
 const ENIG_TRUNCATE    = 0x20;
 const ENIG_DEFAULT_FILE_PERMS = 0600;
 
-const EnigInputStream = new Components.Constructor( "@mozilla.org/scriptableinputstream;1", "nsIScriptableInputStream" );
 
 function EnigCreateFileStream(filePath, permissions) {
   //DEBUG_LOG("enigmailCommon.js: EnigCreateFileStream: file="+filePath+"\n");
@@ -443,7 +446,7 @@ function EnigReadURLContents(url, maxBytes) {
 
   var rawInStream = fileChannel.open();
 
-  var scriptableInStream = new EnigInputStream();
+  var scriptableInStream = Components.classes[ENIG_SCRIPTABLEINPUTSTREAM_CONTRACTID].createInstance(Components.interfaces.nsIScriptableInputStream);
   scriptableInStream.init(rawInStream);
 
   var available = scriptableInStream.available()
@@ -624,7 +627,7 @@ function EnigAdvPrefWindow() {
 
 function EnigHelpWindow(source) {
 
-  input="chrome://enigmail/locale/help/"+source+".html";
+  var input="chrome://enigmail/locale/help/"+source+".html";
 
   EnigOpenWin("enigmail:help",
               "chrome://enigmail/content/enigmailHelp.xul",
@@ -1093,7 +1096,12 @@ function EnigViewDebugLog() {
 
   var logFileURL = "file:///" + logDirectory + "/enigdbug.txt";
 
-  window.open(logFileURL, 'Enigmail Debug Log');
+  EnigOpenWin("enigmail:logFile",
+              "chrome://enigmail/content/enigmailViewFile.xul",
+              "chrome,resizable",
+              [ logFileURL, "Enigmail Debug Log"]);
+
+//  window.open(logFileURL, 'Enigmail Debug Log');
 }
 
 function EnigKeygen() {
@@ -1330,12 +1338,12 @@ function EnigReceiveKey(parent, msgParentWindow, recvFlags, keyId,
   if (progressBar) {
     // wait one second before displaying the progress bar
     var progressParam=Components.classes["@mozilla.org/messengercompose/composeprogressparameters;1"].createInstance(Components.interfaces.nsIMsgComposeProgressParams);
-    parent.setTimeout(progressBar.openProgressDialog, 1000, parent, msgWindow, "chrome://enigmail/content/enigRetrieveProgress.xul", progressParam);
+    parent.setTimeout(progressBar.openProgressDialog, 1000, parent, msgParentWindow, "chrome://enigmail/content/enigRetrieveProgress.xul", progressParam);
     //progressBar.openProgressDialog(parent, msgWindow, "chrome://enigmail/content/enigRetrieveProgress.xul", progressParam);
     progressBar.onStateChange(null, null, Components.interfaces.nsIWebProgressListener.STATE_START, 0);
   }
 
-  return enigmailSvc.receiveKey(keyserver, keyId, requestObserver, errorMsgObj);
+  return enigmailSvc.receiveKey(recvFlags, keyserver, keyId, requestObserver, errorMsgObj);
 }
 
 
@@ -1406,3 +1414,22 @@ function EnigRulesEditor() {
               "dialog,centerscreen,resizable");
 }
 
+// create new PGP Rule
+function EnigNewRule(emailAddress) {    
+  // make sure the rules database is loaded
+  var enigmailSvc = GetEnigmailSvc();
+  if (!enigmailSvc)
+    return false;
+   var rulesListObj= new Object;
+
+  // open rule dialog
+  enigmailSvc.getRulesData(rulesListObj);
+  var inputObj=new Object;
+  var resultObj=new Object;
+  inputObj.toAddress="{"+emailAddress+"}";
+  inputObj.options="";
+  inputObj.command = "add";
+  window.openDialog("chrome://enigmail/content/enigmailSingleRcptSettings.xul","", "dialog,modal,centerscreen,resizable", inputObj, resultObj);
+  return true;
+}
+
