@@ -497,6 +497,7 @@ function enigMessageDecrypt(event) {
 
       var emailAttachment = false;
       var embeddedSigned = null;
+      var embeddedEncrypted = null;
       for (var indexb in currentAttachments) {
         var attachment = currentAttachments[indexb];
         if (attachment.contentType.search(/^message\/rfc822(;|$)/i) == 0) {
@@ -505,6 +506,9 @@ function enigMessageDecrypt(event) {
         if (attachment.contentType.search(/^application\/pgp-signature/i) == 0) {
           embeddedSigned = attachment.url.replace(/\.\d+\.\d+$/, "");
         }
+        if (attachment.contentType.search(/^application\/pgp-encrypted/i) == 0) {
+          embeddedEncrypted = attachment.url.replace(/\.\d+\.\d+$/, "");
+        }
         DEBUG_LOG("enigmailMessengerOverlay.js: "+indexb+": "+attachment.contentType+"\n");
         //DEBUG_LOG("enigmailMessengerOverlay.js: "+indexb+": "+attachment.url+"\n");
       }
@@ -533,7 +537,8 @@ function enigMessageDecrypt(event) {
     xEnigmailVersion = gEnigSavedHeaders["x-enigmail-version"];
   }
 
-  if (contentType.search(/^multipart\/encrypted(;|$)/i) == 0
+  if ((contentType.search(/^multipart\/encrypted(;|$)/i) == 0) ||
+      (embeddedEncrypted && contentType.search(/^multipart\/mixed(;|$)/i) == 0)
       /* && (!embeddedSigned) */ ) {
     // multipart/encrypted
     DEBUG_LOG("enigmailMessengerOverlay.js: multipart/encrypted\n");
@@ -602,7 +607,7 @@ function enigMessageDecrypt(event) {
     }
   }
 
-  return enigMessageParse(!event, false, contentEncoding);
+  enigMessageParse(!event, false, contentEncoding);
 }
 
 
@@ -1604,14 +1609,14 @@ function enigDecryptAttachmentCallback(callbackArg, ctxt) {
       outFile.initWithPath(tmpDir);
       if (!(outFile.isDirectory() && outFile.isWritable())) {
         errorMsgObj.value=EnigGetString("noTempDir");
-        return -1;
+        return;
       }
       outFile.append(rawFileName);
       outFile.createUnique(Components.interfaces.NORMAL_FILE_TYPE, 0600);
     }
     catch (ex) {
       errorMsgObj.value=EnigGetString("noTempDir");
-      return -1;
+      return;
     }
   }
 
@@ -1781,7 +1786,7 @@ function enigHandleUnknownKey() {
 
   if (EnigConfirm(mesg)) {
     var recvErrorMsgObj = new Object();
-    var recvFlags = nsIEnigmail.UI_INTERACTIVE;
+    var recvFlags = nsIEnigmail.DOWNLOAD_KEY;
     var progressBar=Components.classes["@mozilla.org/messenger/progress;1"].createInstance(Components.interfaces.nsIMsgProgress);
     var requestObserver = new EnigRequestObserver(enigReceiveKeyTerminate, {'progressBar': progressBar, 'callType': 1});
 
