@@ -19,18 +19,6 @@ const ENIG_ENIGMSGCOMPFIELDS_CONTRACTID = "@mozdev.org/enigmail/composefields;1"
 // List of hash algorithms for PGP/MIME signatures
 var gMimeHashAlgorithms = ["md5", "sha1", "ripemd160"];
 
-var gSendFlagsObj = {
-  "cmd_sendButton":          0,
-  "cmd_send":                0,
-  "cmd_sendNow":             0,
-  "cmd_sendWithCheck":       nsIEnigmail.SEND_WITH_CHECK,
-  "cmd_sendLater":           nsIEnigmail.SEND_LATER,
-  "cmd_saveAsDraft":         nsIEnigmail.SAVE_MESSAGE,
-  "cmd_saveDefault":         nsIEnigmail.SAVE_MESSAGE,
-  "cmd_saveAsTemplate":      nsIEnigmail.SAVE_MESSAGE
-};
-
-
 var gEnigEditor;
 var gEnigDirty, gEnigProcessed, gEnigTimeoutID;
 var gEnigSendPGPMime, gEnigModifiedAttach, gEnigSendMode;
@@ -38,11 +26,23 @@ var gEnigSendModeDirty = 0;
 var gEnigNextCommand;
 var gEnigAccountId = null;
 
-window.addEventListener("load", enigMsgComposeStartup, false);
+if (typeof(GenericSendMessage)=="function") {
+  // replace GenericSendMessage with our own version
+  var origGenericSendMessage = GenericSendMessage;
+  GenericSendMessage = function (msgType) {
+    enigGenericSendMessage(msgType);
+  }
 
-// Handle recycled windows
-window.addEventListener('compose-window-close', enigMsgComposeClose, true);
-window.addEventListener('compose-window-reopen', enigMsgComposeReopen, true);
+  window.addEventListener("load", enigMsgComposeStartup, false);
+
+  // Handle recycled windows
+  window.addEventListener('compose-window-close', enigMsgComposeClose, true);
+  window.addEventListener('compose-window-reopen', enigMsgComposeReopen, true);
+
+}
+else {
+  EnigAlert("WARNING:\n\nCannot hook message sending -- you will not be able to send OpenPGP messages!");
+}
 
 function enigMsgComposeStartup() {
   DEBUG_LOG("enigmailMsgComposeOverlay.js: enigMsgComposeStartup\n");
@@ -61,17 +61,6 @@ function enigMsgComposeStartup() {
           smimeButton.setAttribute("collapsed", "true");
     }
   }
-  // Override send command
-  var sendElementIds = ["cmd_sendButton", "cmd_sendNow", "cmd_sendWithCheck",
-                        "cmd_sendLater", "cmd_saveAsDraft", "cmd_saveDefault",
-                        "cmd_saveAsTemplate"];
-
-  EnigOverrideAttribute( sendElementIds, "oncommand",
-                         "enigSendCommand('", "');");
-
-
-  EnigOverrideAttribute( ["msgcomposeWindow"], "onclose",
-                        "return enigDoCommandClose('", "')");
 
   var composeStateListener = new EnigComposeStateListener();
 
@@ -131,6 +120,7 @@ function enigGetAccDefault(value) {
       return false;
     }
   }
+  return null;
 }
 
 function enigSetIdentityDefaults() {
@@ -152,7 +142,7 @@ function enigSetIdentityDefaults() {
 function enigSetSendDefaultOptions() {
   gEnigSendMode = 0;
   if (! enigGetAccDefault("enabled")) {
-    return 0;
+    return;
   }
   if (enigGetAccDefault("encrypt")>0) {
     gEnigSendMode |= ENIG_ENCRYPT;
@@ -305,9 +295,8 @@ function enigInsertKey() {
   EnigEditorInsertText(EnigGetString("pubKey",userIdValue) + keyBlock);
 }
 
-function enigUndoEncryption( bucketList, modifiedAttachments ) {
+function enigUndoEncryption() {
   DEBUG_LOG("enigmailMsgComposeOverlay.js: enigUndoEncryption: \n");
-
   if (gEnigProcessed) {
     enigReplaceEditorText(gEnigProcessed.origText);
 
@@ -317,13 +306,14 @@ function enigUndoEncryption( bucketList, modifiedAttachments ) {
     enigDecryptQuote(true);
   }
 
-  if ( modifiedAttachments && bucketList && bucketList.hasChildNodes() ) {
+  var bucketList = document.getElementById("attachmentBucket");
+  if ( gEnigModifiedAttach && bucketList && bucketList.hasChildNodes() ) {
     // undo inline encryption of attachments
-    for (var i=0; i<modifiedAttachments.length; i++) {
+    for (var i=0; i<gEnigModifiedAttach.length; i++) {
       var node = bucketList.firstChild;
       while (node) {
-        if (node.attachment.url == modifiedAttachments[i].newUrl) {
-          if (modifiedAttachments[i].dummyFile) {
+        if (node.attachment.url == gEnigModifiedAttach[i].newUrl) {
+          if (gEnigModifiedAttach[i].dummyFile) {
             node.attachment=null;
             var delNode=node;
             node=node.nextSibling;
@@ -331,14 +321,14 @@ function enigUndoEncryption( bucketList, modifiedAttachments ) {
             continue;
           }
           else {
-            node.attachment.url = modifiedAttachments[i].origUrl;
-            node.attachment.name = modifiedAttachments[i].origName;
-            node.attachment.temporary = modifiedAttachments[i].origTemp;
-            node.attachment.contentType = modifiedAttachments[i].origCType;
+            node.attachment.url = gEnigModifiedAttach[i].origUrl;
+            node.attachment.name = gEnigModifiedAttach[i].origName;
+            node.attachment.temporary = gEnigModifiedAttach[i].origTemp;
+            node.attachment.contentType = gEnigModifiedAttach[i].origCType;
           }
           // delete encrypted file
           try {
-            modifiedAttachments[i].newFile.remove(false);
+            gEnigModifiedAttach[i].newFile.remove(false);
           }
           catch (ex) {}
         }
@@ -346,7 +336,7 @@ function enigUndoEncryption( bucketList, modifiedAttachments ) {
       }
     }
 
-    modifiedAttachments = null;
+    gEnigModifiedAttach = null;
   }
 }
 
@@ -551,7 +541,7 @@ function enigDisplaySignClickWarn() {
   }
 }
 
-function enigConfirmBeforeSend(toAddr, gpgKeys, sendFlags, isOffline) {
+function enigConfirmBeforeSend(toAddr, gpgKeys, sendFlags, isOffline, msgSendType) {
   // get confirmation before sending message
   var msgStatus = "";
 
@@ -570,20 +560,38 @@ function enigConfirmBeforeSend(toAddr, gpgKeys, sendFlags, isOffline) {
   }
 
   var msgConfirm = (isOffline || sendFlags & nsIEnigmail.SEND_LATER)
-          ? EnigGetString("offlineSave",msgStatus,EnigStripEmail(toAddr))
-          : EnigGetString("onlineSend",msgStatus,EnigStripEmail(toAddr));
+          ? EnigGetString("offlineSave",msgStatus,EnigStripEmail(toAddr).replace(/,/g, ", "))
+          : EnigGetString("onlineSend",msgStatus,EnigStripEmail(toAddr).replace(/,/g, ", "));
   if (sendFlags & ENIG_ENCRYPT)
     msgConfirm += "\n\n"+EnigGetString("encryptKeysNote", gpgKeys);
 
   return EnigConfirm(msgConfirm);
 }
 
-function enigSendCommand(elementId) {
-  DEBUG_LOG("enigmailMsgComposeOverlay.js: enigSendCommand: id="+elementId+", gEnigSendMode="+gEnigSendMode+"\n");
+function enigAddRecipients(toAddrList, recList) {
+  for (var i=0; i<recList.count; i++) {
+    toAddrList.push(EnigStripEmail(recList.StringAt(i).replace(/[\",]/g, "")));
+  }
+}
+
+function enigEncryptMsg(msgSendType) {
+  DEBUG_LOG("enigmailMsgComposeOverlay.js: enigEncryptMsg: msgType="+msgSendType+", gEnigSendMode="+gEnigSendMode+"\n");
 
   var gotSendFlags = gEnigSendMode;
   var sendFlags=0;
-  sendFlags |= gSendFlagsObj[elementId];
+  window.enigmailSendFlags=0;
+  switch (msgSendType) {
+  case nsIMsgCompDeliverMode.Later:          
+    sendFlags |= nsIEnigmail.SEND_LATER;
+    break;
+  case nsIMsgCompDeliverMode.SaveAsDraft:   
+    sendFlags |=  nsIEnigmail.SAVE_MESSAGE;
+    break;
+  case nsIMsgCompDeliverMode.SaveAsTemplate: 
+    sendFlags |= nsIEnigmail.SAVE_MESSAGE;
+    break;
+  }
+    
   if (gotSendFlags & ENIG_SIGN)
       sendFlags |= ENIG_SIGN;
   if (gotSendFlags & ENIG_ENCRYPT) {
@@ -593,13 +601,13 @@ function enigSendCommand(elementId) {
   if (sendFlags & nsIEnigmail.SAVE_MESSAGE) {
     if (!((sendFlags & ENIG_ENCRYPT) && EnigConfirmPref(EnigGetString("savingMessage"), "saveEncrypted"))) {
       sendFlags &= ~ENIG_ENCRYPT;
-      goDoCommand(elementId);
       return;
     }
   }
 
   if (gWindowLocked) {
     EnigAlert(EnigGetString("windowLocked"));
+    window.cancelSendMessage=true;
     return;
   }
 
@@ -637,9 +645,8 @@ function enigSendCommand(elementId) {
         msg = gEnigmailSvc.initializationError +"\n\n"+msg;
      }
 
-     if (EnigConfirm(msg))
-        goDoCommand(elementId);
-
+     if (!EnigConfirm(msg))
+        window.cancelSendMessage=true;
      return;
   }
 
@@ -650,7 +657,7 @@ function enigSendCommand(elementId) {
      var errorMsgObj    = new Object();
      gEnigModifiedAttach = null;
 
-     DEBUG_LOG("enigmailMsgComposeOverlay.js: enigSendCommand: currentId="+gEnigAccountId+
+     DEBUG_LOG("enigmailMsgComposeOverlay.js: enigEncryptMsg: currentId="+gEnigAccountId+
               ", "+gEnigAccountId.email+"\n");
      var fromAddr = gEnigAccountId.email;
 
@@ -658,11 +665,8 @@ function enigSendCommand(elementId) {
 
      if (! pgpEnabled) {
         if (sendFlags & ENIG_ENCRYPT_OR_SIGN) {
-          if (EnigConfirm(EnigGetString("acctNotConfigured")))
-              goDoCommand(elementId);
-        }
-        else {
-          goDoCommand(elementId);
+          if (!EnigConfirm(EnigGetString("acctNotConfigured")))
+              window.cancelSendMessage=true;
         }
         return;
      }
@@ -707,45 +711,45 @@ function enigSendCommand(elementId) {
      }
 
      var msgCompFields = gMsgCompose.compFields;
-     Recipients2CompFields(msgCompFields);
 
      // Check if sending to any newsgroups
      var newsgroups = msgCompFields.newsgroups;
 
-     DEBUG_LOG("enigmailMsgComposeOverlay.js: enigSendCommand:gMsgCompose="+gMsgCompose+"\n");
-
-     try {
-       var dummy = new Object();
-       gMsgCompose.CheckAndPopulateRecipients(true, false, dummy);
-     }
-     catch (ex) {}
+     DEBUG_LOG("enigmailMsgComposeOverlay.js: enigEncryptMsg:gMsgCompose="+gMsgCompose+"\n");
 
      var toAddrList = [];
+     if (msgCompFields.to) {
+       var recList = msgCompFields.SplitRecipients(msgCompFields.to, true)
+       enigAddRecipients(toAddrList, recList);
+     }
 
-     if (msgCompFields.to)  toAddrList.push(msgCompFields.to);
-
-     if (msgCompFields.cc)  toAddrList.push(msgCompFields.cc);
+     if (msgCompFields.cc) {
+       recList = msgCompFields.SplitRecipients(msgCompFields.cc, true)
+       enigAddRecipients(toAddrList, recList);
+     }
 
      if (msgCompFields.bcc) {
-       toAddrList.push(msgCompFields.bcc);
+       recList = msgCompFields.SplitRecipients(msgCompFields.bcc, true)
+       enigAddRecipients(toAddrList, recList);
 
        var bccLC = EnigStripEmail(msgCompFields.bcc).toLowerCase()
-       DEBUG_LOG("enigmailMsgComposeOverlay.js: enigSendCommand: BCC: "+bccLC+"\n");
+       DEBUG_LOG("enigmailMsgComposeOverlay.js: enigEncryptMsg: BCC: "+bccLC+"\n");
 
-       var selfBCC = fromAddr && (fromAddr.toLowerCase() == bccLC);
+       var selfBCC = gEnigAccountId.email && (gEnigAccountId.email.toLowerCase() == bccLC);
 
        if (selfBCC) {
-         DEBUG_LOG("enigmailMsgComposeOverlay.js: enigSendCommand: Self BCC\n");
+         DEBUG_LOG("enigmailMsgComposeOverlay.js: enigEncryptMsg: Self BCC\n");
 
        } else if (sendFlags & ENIG_ENCRYPT) {
          // BCC and encryption
 
          if (encryptIfPossible) {
            sendFlags &= ~ENIG_ENCRYPT;
-           DEBUG_LOG("enigmailMsgComposeOverlay.js: enigSendCommand: No default encryption because of BCC\n");
+           DEBUG_LOG("enigmailMsgComposeOverlay.js: enigEncryptMsg: No default encryption because of BCC\n");
 
          } else {
            if (!EnigConfirm(EnigGetString("sendingBCC"))) {
+             window.cancelSendMessage=true;
              return;
            }
          }
@@ -759,11 +763,12 @@ function enigSendCommand(elementId) {
 
          if (!encryptIfPossible) {
            EnigAlert(EnigGetString("sendingNews"));
+           window.cancelSendMessage=true;
            return;
          }
 
          sendFlags &= ~ENIG_ENCRYPT;
-         DEBUG_LOG("enigmailMsgComposeOverlay.js: enigSendCommand: No default encryption because of newsgroups\n");
+         DEBUG_LOG("enigmailMsgComposeOverlay.js: enigEncryptMsg: No default encryption because of newsgroups\n");
 
        }
      }
@@ -782,7 +787,7 @@ function enigSendCommand(elementId) {
 
      if (toAddr.length>=1) {
 
-        DEBUG_LOG("enigmailMsgComposeOverlay.js: enigSendCommand: toAddr="+toAddr+"\n");
+        DEBUG_LOG("enigmailMsgComposeOverlay.js: enigEncryptMsg: toAddr="+toAddr+"\n");
         var perRecipientRules=EnigGetPref("perRecipientRules");
         var repeatSelection=0;
         while (repeatSelection<2) {
@@ -793,6 +798,7 @@ function enigSendCommand(elementId) {
                                   (repeatSelection==1),
                                   matchedKeysObj, 
                                   flagsObj)) {
+              window.cancelSendMessage=true;
               return;
             }
            
@@ -874,7 +880,10 @@ function enigSendCommand(elementId) {
   
                 window.openDialog("chrome://enigmail/content/enigmailUserSelection.xul","", "dialog,modal,centerscreen", inputObj, resultObj);
                 try {
-                  if (resultObj.cancelled) return;
+                  if (resultObj.cancelled) {
+                    window.cancelSendMessage=true;
+                    return;
+                  }
                   if (resultObj.perRecipientRules) {
                     // do an extra round because the user want to set a PGP rule
                     continue;
@@ -890,13 +899,15 @@ function enigSendCommand(elementId) {
                   testExitCodeObj.value = 0;
                 } catch (ex) {
                   // cancel pressed -> don't send mail
+                  EnigAlert("Something went wrong 1"); // REMOVE
+                  window.cancelSendMessage=true;
                   return;
                 }
             }
             if ((!testCipher || (testExitCodeObj.value != 0)) && recipientsSelectionOption==0) {
                 // Test encryption failed; turn off default encryption
                 sendFlags &= ~ENIG_ENCRYPT;
-                DEBUG_LOG("enigmailMsgComposeOverlay.js: enigSendCommand: No default encryption because test failed\n");
+                DEBUG_LOG("enigmailMsgComposeOverlay.js: enigEncryptMsg: No default encryption because test failed\n");
             }
           }
           repeatSelection=2;
@@ -911,26 +922,6 @@ function enigSendCommand(elementId) {
         }
      }
 
-     if (!gEnigProcessed) {
-/////////////////////////////////////////////////////////////////////////
-// The following spellcheck logic is from the function
-// GenericSendMessage from the file MsgComposeCommands.js
-/////////////////////////////////////////////////////////////////////////
-       if (gEnigPrefRoot.getBoolPref("mail.SpellCheckBeforeSend")) {
-        // We disable spellcheck for the following -subject line, attachment pane, identity and addressing widget
-        // therefore we need to explicitly focus on the mail body when we have to do a spellcheck.
-         window.content.focus();
-
-         window.cancelSendMessage = false;
-         try {
-           window.openDialog("chrome://editor/content/EdSpellCheck.xul", "_blank", "chrome,close,titlebar,modal", true, true, false);
-         } catch(ex){}
-
-         if(window.cancelSendMessage)
-           return;
-       }
-     }
-
      var bucketList = document.getElementById("attachmentBucket");
      var hasAttachments = bucketList && bucketList.hasChildNodes();
 
@@ -949,7 +940,7 @@ function enigSendCommand(elementId) {
         !(sendFlags & nsIEnigmail.SEND_PGP_MIME) &&
         enigmailSvc.composeSecure) {
 
-        var inputObj = new Object();
+        inputObj = new Object();
         inputObj.pgpMimePossible = (usePGPMimeOption >= PGP_MIME_POSSIBLE);
         inputObj.inlinePossible = (sendFlags & ENIG_ENCRYPT); // makes no sense for sign only!
 
@@ -964,11 +955,12 @@ function enigSendCommand(elementId) {
         }
 
         if (inputObj.pgpMimePossible || inputObj.inlinePossible) {
-          var resultObj = new Object();
+          resultObj = new Object();
           resultObj.selected = -1;
           window.openDialog("chrome://enigmail/content/enigmailAttachmentsDialog.xul","", "dialog,modal,centerscreen", inputObj, resultObj);
           if (resultObj.selected < 0) {
             // dialog cancelled
+            window.cancelSendMessage=true;
             return;
           }
           else if (resultObj.selected == 1) {
@@ -983,6 +975,7 @@ function enigSendCommand(elementId) {
         else {
           if (sendFlags & ENIG_ENCRYPT) {
             if (!EnigConfirm(EnigGetString("attachWarning")))
+              window.cancelSendMessage=true;
               return;
           }
         }
@@ -1007,13 +1000,11 @@ function enigSendCommand(elementId) {
 
      if ((sendFlags & ENIG_ENCRYPT_OR_SIGN) && usingPGPMime) {
        // Use EnigMime
-       DEBUG_LOG("enigmailMsgComposeOverlay.js: enigSendCommand: Using EnigMime, flags="+sendFlags+"\n");
+       DEBUG_LOG("enigmailMsgComposeOverlay.js: enigEncryptMsg: Using EnigMime, flags="+sendFlags+"\n");
 
        var oldSecurityInfo = gMsgCompose.compFields.securityInfo;
 
-       DEBUG_LOG("enigmailMsgComposeOverlay.js: enigSendCommand: oldSecurityInfo = "+oldSecurityInfo+"\n");
-
-       var newSecurityInfo;
+       DEBUG_LOG("enigmailMsgComposeOverlay.js: enigEncryptMsg: oldSecurityInfo = "+oldSecurityInfo+"\n");
 
        if (!oldSecurityInfo) {
          try {
@@ -1037,7 +1028,7 @@ function enigSendCommand(elementId) {
        newSecurityInfo.recipients = toAddr;
        newSecurityInfo.hashAlgorithm = gMimeHashAlgorithms[EnigGetPref("mimeHashAlgorithm")];
 
-       DEBUG_LOG("enigmailMsgComposeOverlay.js: enigSendCommand: securityInfo = "+newSecurityInfo+"\n");
+       DEBUG_LOG("enigmailMsgComposeOverlay.js: enigEncryptMsg: securityInfo = "+newSecurityInfo+"\n");
 
      } else if (!gEnigProcessed && (sendFlags & ENIG_ENCRYPT_OR_SIGN)) {
 
@@ -1051,6 +1042,7 @@ function enigSendCommand(elementId) {
          var convert = DetermineConvertibility();
          if (convert == nsIMsgCompConvertible.No) {
            if (!EnigConfirm(EnigGetString("strippingHTML")))
+             window.cancelSendMessage=true;
              return;
          }
        } catch (ex) {
@@ -1192,6 +1184,7 @@ function enigSendCommand(elementId) {
            if (sendFlags & ENIG_ENCRYPT_OR_SIGN) {
              // Encryption/signing failed
              EnigAlert(EnigGetString("sendAborted")+errorMsgObj.value);
+             window.cancelSendMessage=true;
              return;
            }
          }
@@ -1199,7 +1192,7 @@ function enigSendCommand(elementId) {
          if (inlineEncAttach) {
             // encrypt attachments
             gEnigModifiedAttach = new Array();
-            var exitCode = enigEncryptAttachments(bucketList, gEnigModifiedAttach,
+            exitCode = enigEncryptAttachments(bucketList, gEnigModifiedAttach,
                                     window, uiFlags, fromAddr, toAddr, sendFlags,
                                     errorMsgObj);
             if (exitCode != 0) {
@@ -1211,7 +1204,8 @@ function enigSendCommand(elementId) {
                 EnigAlert(EnigGetString("sendAborted")+"an internal error has occurred");
               }
               if (gEnigProcessed)
-                enigUndoEncryption(bucketList, gEnigModifiedAttach);
+                enigUndoEncryption();
+              window.cancelSendMessage=true;
               return;
             }
          }
@@ -1220,58 +1214,49 @@ function enigSendCommand(elementId) {
 
      // EnigSend: Handle both plain and encrypted messages below
      var isOffline = (gIOService && gIOService.offline);
+     window.enigmailSendFlags=sendFlags;
 
+     
      if (sendFlags & nsIEnigmail.SAVE_MESSAGE) {
-       goDoCommand(elementId);
-       //if (! (sendFlags & nsIEnigmail.SEND_PGP_MIME)) {
-          enigUndoEncryption(bucketList, gEnigModifiedAttach);
-       //}
-
-       return;
+       // update the list of attachments
+       Attachments2CompFields(msgCompFields);
      }
-
-     if (EnigGetPref("confirmBeforeSend")) {
+     else if (EnigGetPref("confirmBeforeSend")) {
        if (!enigConfirmBeforeSend(toAddrList.join(", "), toAddr, sendFlags, isOffline)) {
          if (gEnigProcessed)
-           enigUndoEncryption(bucketList, gEnigModifiedAttach);
-
+           enigUndoEncryption();
+           
+         window.cancelSendMessage=true;
          return;
        }
-
-     } else if (isOffline &&
+     } 
+     else if (isOffline &&
                 !EnigConfirm(EnigGetString("offlineNote")) ) {
        // Abort send
        if (gEnigProcessed)
-         enigUndoEncryption(bucketList, gEnigModifiedAttach);
-
+         enigUndoEncryption();
+         
+       window.cancelSendMessage=true;
        return;
 
-     } else if ( (sendFlags & nsIEnigmail.SEND_WITH_CHECK) &&
+     }
+     else if ( (sendFlags & nsIEnigmail.SEND_WITH_CHECK) &&
                  !enigMessageSendCheck() ) {
        // Abort send
        if (gEnigProcessed)
-         enigUndoEncryption(bucketList, gEnigModifiedAttach);
+         enigUndoEncryption();
 
+       window.cancelSendMessage=true;
        return;
      }
 
-     if (isOffline || (sendFlags & nsIEnigmail.SEND_LATER)) {
-       // Send message later
-       DEBUG_LOG("enigmailMsgComposeOverlay.js: Sending message later ...\n");
-
-       enigGenericSendMessage(nsIMsgCompDeliverMode.Later);
-       return;
-     }
-
-     enigGenericSendMessage(nsIMsgCompDeliverMode.Now);
-
   } catch (ex) {
-     var msg=EnigGetString("signFailed");
+     msg=EnigGetString("signFailed");
      if (gEnigmailSvc && gEnigmailSvc.initializationError) {
         msg += "\n"+gEnigmailSvc.initializationError;
      }
-     if (EnigConfirm(msg))
-       goDoCommand(elementId);
+     if (!EnigConfirm(msg))
+       window.cancelSendMessage=true;
   }
 }
 
@@ -1354,10 +1339,19 @@ function enigGenericSendMessage( msgType )
       if (msgType == nsIMsgCompDeliverMode.Now || msgType == nsIMsgCompDeliverMode.Later)
       {
         //Do we need to check the spelling?
-/////////////////////////////////////////////////////////////////////////
-// MODIFICATION
-//      Moved spell check logic to enigSendCommand
-/////////////////////////////////////////////////////////////////////////
+        if (sPrefs.getBoolPref("mail.SpellCheckBeforeSend")){
+        //We disable spellcheck for the following -subject line, attachment pane, identity and addressing widget
+        //therefore we need to explicitly focus on the mail body when we have to do a spellcheck.
+          window.content.focus();
+          window.cancelSendMessage = false;
+          try {
+            window.openDialog("chrome://editor/content/EdSpellCheck.xul", "_blank",
+                    "chrome,close,titlebar,modal", true, true, false);
+          }
+          catch(ex){}
+          if(window.cancelSendMessage)
+            return;
+        }            
 
         //Check if we have a subject, else ask user for confirmation
         if (subject == "")
@@ -1380,23 +1374,23 @@ function enigGenericSendMessage( msgType )
               }
               else
                 return;
-            }
           }
+        }
 
         // Before sending the message, check what to do with HTML message, eventually abort.
         var convert = DetermineConvertibility();
         var action = DetermineHTMLAction(convert);
         if (action == nsIMsgCompSendFormat.AskUser)
         {
-                    var recommAction = convert == nsIMsgCompConvertible.No
-                                   ? nsIMsgCompSendFormat.AskUser
-                                   : nsIMsgCompSendFormat.PlainText;
-                    var result2 = {action:recommAction,
-                                  convertible:convert,
-                                  abort:false};
-                    window.openDialog("chrome://messenger/content/messengercompose/askSendFormat.xul",
-                                      "askSendFormatDialog", "chrome,modal,titlebar,centerscreen",
-                                      result2);
+          var recommAction = convert == nsIMsgCompConvertible.No
+                         ? nsIMsgCompSendFormat.AskUser
+                         : nsIMsgCompSendFormat.PlainText;
+          var result2 = {action:recommAction,
+                        convertible:convert,
+                        abort:false};
+          window.openDialog("chrome://messenger/content/messengercompose/askSendFormat.xul",
+                            "askSendFormatDialog", "chrome,modal,titlebar,centerscreen",
+                            result2);
           if (result2.abort)
             return;
           action = result2.action;
@@ -1422,22 +1416,31 @@ function enigGenericSendMessage( msgType )
       // hook for extra compose pre-processing
       var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
       observerService.notifyObservers(window, "mail:composeOnSend", null);
-
+/*
+/////////////////////////////////////////////////////////////////////////
+// MODIFICATION
+// Set the charset from the original text
+// (after setting observerService)
+/////////////////////////////////////////////////////////////////////////
       if (gEnigProcessed) {
         // Ensure that original charset is preserved for encrypted messages
         gMsgCompose.SetDocumentCharset(gEnigProcessed.charset);
 
       } else {
-        // Check if the headers of composing mail can be converted to a mail charset.
+/////////////////////////////////////////////////////////////////////////
+// END OF MODIFICATION
+/////////////////////////////////////////////////////////////////////////
+*/
+         // Check if the headers of composing mail can be converted to a mail charset.
         if (msgType == nsIMsgCompDeliverMode.Now ||
-          msgType == nsIMsgCompDeliverMode.Later ||
-          msgType == nsIMsgCompDeliverMode.Save ||
-          msgType == nsIMsgCompDeliverMode.SaveAsDraft ||
-          msgType == nsIMsgCompDeliverMode.SaveAsTemplate)
+            msgType == nsIMsgCompDeliverMode.Later ||
+            msgType == nsIMsgCompDeliverMode.Save ||
+            msgType == nsIMsgCompDeliverMode.SaveAsDraft ||
+            msgType == nsIMsgCompDeliverMode.SaveAsTemplate)
         {
           var fallbackCharset = new Object;
           if (gPromptService &&
-              !gMsgCompose.checkCharsetConversion(gEnigAccountId, fallbackCharset))
+              !gMsgCompose.checkCharsetConversion(getCurrentIdentity(), fallbackCharset))
           {
             var dlgTitle = sComposeMsgsBundle.getString("initErrorDlogTitle");
             var dlgText = sComposeMsgsBundle.getString("12553");  // NS_ERROR_MSG_MULTILINGUAL_SEND
@@ -1449,7 +1452,24 @@ function enigGenericSendMessage( msgType )
               fallbackCharset.value && fallbackCharset.value != "")
             gMsgCompose.SetDocumentCharset(fallbackCharset.value);
         }
-      }
+//      }
+
+/////////////////////////////////////////////////////////////////////////
+// MODIFICATION
+// Call the following function from our version of the function
+// GenericSendMessage from the file MsgComposeCommands.js
+// (after having determined HTML convertibility)
+/////////////////////////////////////////////////////////////////////////
+          try {
+            window.cancelSendMessage=false;
+            enigEncryptMsg(msgType);
+            if(window.cancelSendMessage)
+              return;
+          }
+          catch(ex){}
+/////////////////////////////////////////////////////////////////////////
+// END OF MODIFICATION
+/////////////////////////////////////////////////////////////////////////
 
       try {
         gWindowLocked = true;
@@ -1462,20 +1482,25 @@ function enigGenericSendMessage( msgType )
           progress.registerListener(progressListener);
           gSendOrSaveOperationInProgress = true;
         }
-        try {
-          // Mozilla <= 1.5
-          gMsgCompose.SendMsg(msgType, gEnigAccountId, progress);
+        msgWindow.SetDOMWindow(window);
+        
+        if (typeof(getCurrentAccountKey)=="function") {
+          // Mozilla >= 1.7a
+          gMsgCompose.SendMsg(msgType, getCurrentIdentity(), getCurrentAccountKey(), msgWindow, progress);
         }
-        catch (ex) {
-          msgWindow.SetDOMWindow(window);
-          if (typeof(getCurrentAccountKey)=="function") {
-            // Mozilla >= 1.7a
-            gMsgCompose.SendMsg(msgType, getCurrentIdentity(), getCurrentAccountKey(), msgWindow, progress);
-          }
-          else {
-            //Mozilla 1.6
-            gMsgCompose.SendMsg(msgType, getCurrentIdentity(), msgWindow, progress);
-          }
+        else {
+          //Mozilla <= 1.6
+          gMsgCompose.SendMsg(msgType, getCurrentIdentity(), msgWindow, progress);
+        }
+        
+/////////////////////////////////////////////////////////////////////////
+// MODIFICATION
+// Call the following function from our version of the function
+// GenericSendMessage from the file MsgComposeCommands.js
+// (after gMsgCompose.SendMsg)
+/////////////////////////////////////////////////////////////////////////
+        if (window.enigmailSendFlags & nsIEnigmail.SAVE_MESSAGE) {
+          enigUndoEncryption();
         }
       }
       catch (ex) {
@@ -1490,99 +1515,6 @@ function enigGenericSendMessage( msgType )
     dump("###SendMessage Error: composeAppCore is null!\n");
 }
 
-// Modified version of DoCommandClose() function from MsgComposeCommands.js
-function enigDoCommandClose() {
-  var retVal;
-
-// MODIFICATION
-  if ((retVal = enigComposeCanClose())) {
-
-    // Notify the SendListener that Send has been aborted and Stopped
-    if (gMsgCompose)
-    {
-      var externalListener = gMsgCompose.getExternalSendListener();
-      if (externalListener)
-      {
-        externalListener.onSendNotPerformed(null, Components.results.NS_ERROR_ABORT);
-      }
-    }
-
-    MsgComposeCloseWindow(true);
-
-    // at this point, we might be caching this window.
-    // in which case, we don't want to close it
-    if (sMsgComposeService.isCachedWindow(window)) {
-      retVal = false;
-    }
-  }
-
-  return retVal;
-}
-
-// modified version of ComposeCanClose from MsgComposeCommands.js
-function enigComposeCanClose()
-{
-  if (gSendOrSaveOperationInProgress)
-  {
-    var result;
-
-    if (gPromptService)
-    {
-      var promptTitle = sComposeMsgsBundle.getString("quitComposeWindowTitle");
-      var promptMsg = sComposeMsgsBundle.getString("quitComposeWindowMessage");
-      var quitButtonLabel = sComposeMsgsBundle.getString("quitComposeWindowQuitButtonLabel");
-      var waitButtonLabel = sComposeMsgsBundle.getString("quitComposeWindowWaitButtonLabel");
-
-      result = gPromptService.confirmEx(window, promptTitle, promptMsg,
-          (gPromptService.BUTTON_TITLE_IS_STRING*gPromptService.BUTTON_POS_0) +
-          (gPromptService.BUTTON_TITLE_IS_STRING*gPromptService.BUTTON_POS_1),
-          waitButtonLabel, quitButtonLabel, null, null, {value:0});
-
-      if (result == 1)
-      {
-        gMsgCompose.abort();
-        return true;
-      }
-      return false;
-    }
-  }
-
-  // Returns FALSE only if user cancels save action
-  if (gContentChanged || gMsgCompose.bodyModified)
-  {
-    // call window.focus, since we need to pop up a dialog
-    // and therefore need to be visible (to prevent user confusion)
-    window.focus();
-    if (gPromptService)
-    {
-      result = gPromptService.confirmEx(window,
-                              sComposeMsgsBundle.getString("saveDlogTitle"),
-                              sComposeMsgsBundle.getString("saveDlogMessage"),
-                              (gPromptService.BUTTON_TITLE_SAVE * gPromptService.BUTTON_POS_0) +
-                              (gPromptService.BUTTON_TITLE_CANCEL * gPromptService.BUTTON_POS_1) +
-                              (gPromptService.BUTTON_TITLE_DONT_SAVE * gPromptService.BUTTON_POS_2),
-                              null, null, null,
-                              null, {value:0});
-      switch (result)
-      {
-        case 0: //Save
-          gCloseWindowAfterSave = true;
-/// MODIFICATION
-          enigSendCommand("cmd_saveAsDraft");
-          return false;
-        case 1: //Cancel
-          return false;
-        case 2: //Don't Save
-          break;
-      }
-    }
-
-    SetContentAndBodyAsUnmodified();
-  }
-
-  return true;
-}
-
 
 // encrypt attachments when sending inline PGP mails
 // It's quite a hack: the attachments are stored locally
@@ -1796,7 +1728,7 @@ function enigDecryptQuote(interactive) {
 
   if (tail.search(/\S/) < 0) {
     // No non-space characters in tail; delete it
-    tail = ""
+    tail = "";
   }
 
   //DEBUG_LOG("enigmailMsgComposeOverlay.js: enigDecryptQuote: pgpBlock='"+pgpBlock+"'\n");
@@ -1806,7 +1738,21 @@ function enigDecryptQuote(interactive) {
 
   // Encode ciphertext from unicode to charset
   var cipherText = EnigConvertFromUnicode(pgpBlock, charset);
-
+  
+  if ((! gEnigPrefRoot.getBoolPref("mailnews.reply_in_default_charset")) && (blockType == "MESSAGE")) {
+    // set charset according to PGP block, if available (encrypted messages only)
+    cipherText = cipherText.replace(/\r\n/g, "\n");
+    cipherText = cipherText.replace(/\r/g,   "\n");
+    var cPos = cipherText.search(/\nCharset: .+\n/i);
+    if (cPos < cipherText.search(/\n\n/)) {
+      var charMatch = cipherText.match(/\n(Charset: )(.+)\n/i);
+      if (charMatch && charMatch.length > 2) {
+        charset = charMatch[2];
+        gMsgCompose.SetDocumentCharset(charset);
+      }
+    }
+  }
+  
   // Decrypt message
   var signatureObj   = new Object();
   signatureObj.value = "";
@@ -1859,7 +1805,7 @@ function enigDecryptQuote(interactive) {
                                                   nsIEnigmail.SIGNATURE_TEXT);
   }
 
-  var doubleDashSeparator = ("doubleDashSeparator")
+  var doubleDashSeparator = EnigGetPref("doubleDashSeparator")
   if (doubleDashSeparator) {
     var signOffset = plainText.search(/[\r\n]-- +[\r\n]/);
 
@@ -1869,7 +1815,22 @@ function enigDecryptQuote(interactive) {
     }
   }
   
-  // Replace encrypted quote with decrypted quote
+  var clipBoard = Components.classes[ENIG_CLIPBOARD_CONTRACTID].getService(Components.interfaces.nsIClipboard);
+  if (clipBoard.supportsSelectionClipboard()) {
+    // get the clipboard contents for selected text (X11)
+    try {
+      var transferable = Components.classes[ENIG_TRANSFERABLE_CONTRACTID].createInstance(Components.interfaces.nsITransferable);
+      transferable.addDataFlavor("text/unicode");
+      clipBoard.getData(transferable, clipBoard.kSelectionClipboard);
+      var flavour = {};
+      var data = {};
+      var length = {};
+      transferable.getAnyTransferData(flavour, data, length);
+    }
+    catch(ex) {}
+  }
+  
+  // Replace encrypted quote with decrypted quote (destroys selection clipboard on X11)
   EnigEditorSelectAll();
 
   //DEBUG_LOG("enigmailMsgComposeOverlay.js: enigDecryptQuote: plainText='"+plainText+"'\n");
@@ -1888,6 +1849,16 @@ function enigDecryptQuote(interactive) {
 
   if (tail)
     EnigEditorInsertText(tail);
+    
+  if (clipBoard.supportsSelectionClipboard()) {
+    try {
+      // restore the clipboard contents for selected text (X11)
+      var pasteClipboard = Components.classes[ENIG_CLIPBOARD_HELPER_CONTRACTID].getService(Components.interfaces.nsIClipboardHelper);
+      data = data.value.QueryInterface(Components.interfaces.nsISupportsString).data;
+      pasteClipboard.copyStringToClipboard(data, clipBoard.kSelectionClipboard);
+    }
+    catch (ex) {}
+  }
 
   if (interactive)
     return;
